﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Sample_cs_2005
{
	public partial class Form1 : Form
	{
		/// <summary>
		/// アプリケーションのメイン エントリ ポイントです。
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.Run(new Form1());
		}

		public Form1()
		{
			InitializeComponent();
		}

		/// <summary>
		/// ＰＯＰクライアントクラス
		/// </summary>
		private TKMP.Net.PopClient PopClient;

		/// <summary>
		/// サーバーへの接続処理
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button1_Click(object sender, EventArgs e)
		{
			if (PopClient != null)
			{
				return;
			}

			//接続用の情報作成
			TKMP.Net.BasicPopLogon bp = new TKMP.Net.BasicPopLogon(txtUserID.Text, txtPassword.Text);
			int Port = 110;
			//ＰＯＰクライアントクラスの定義
			PopClient = new TKMP.Net.PopClient(bp, txtAddress.Text, Port);
			//モニタ表示のために送受信イベントを登録
			PopClient.MessageReceive += new TKMP.Net.MessageReceiveHandler(PopClient_MessageReceive);
			PopClient.MessageSend += new TKMP.Net.MessageSendHandler(PopClient_MessageSend);

			// --------
			//  暗号化
			// --------
			//PopClient.AuthenticationProtocol = TKMP.Net.AuthenticationProtocols.TLS; //TLSでの暗号化通信を行います
			// TKMP.Net.AuthenticationProtocols.SSL を使用する場合は接続ポートを995に変更してください
			//SSLを使用し証明書の検証を行うには次のイベントを使用してください
			//PopClient.CertificateValidation += new TKMP.Net.CertificateValidationHandler(PopClient_CertificateValidation);

			//接続開始
			if (!PopClient.Connect())
			{
				statusBarPanel3.Text = "接続失敗";
				PopClient.Close();
				PopClient = null;
				return;
			}
			else
			{
				statusBarPanel3.Text = "接続成功";
				if (PopClient.IsEncrypted)
				{
					statusBarPanel3.Text += " (通信は暗号化されています)";
				}
			}


			//リストビューへ情報表示
			TKMP.Net.MailData[] Mails = PopClient.MailDatas;
			for (int i = 0; i < Mails.Length; i++)
			{
				statusBarPanel1.Text = "受信数  " + (i + 1).ToString() + " / " + Mails.Length.ToString();
				TKMP.Net.MailData Data = (TKMP.Net.MailData)Mails[i];
				this.listView1.Items.Add(new MailItem(Data));
			}
			//接続を維持する
			PopClient.KeepAlive = true;
		}

		/// <summary>
		/// サーバーの切断処理
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button2_Click(object sender, System.EventArgs e)
		{
			if (PopClient == null)
			{
				return;
			}
			PopClient.Close();
			PopClient = null;
			this.listView1.Items.Clear();
			this.textBox1.Clear();
			statusBarPanel3.Text = "切断";

		}
		/// <summary>
		/// メール送信用のフォームを開きます
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void button3_Click(object sender, System.EventArgs e)
		{
			new Form2().ShowDialog();
		}

		/// <summary>
		/// リストビューをダブルクリックでメール本文の解析を行う
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void listView1_DoubleClick(object sender, System.EventArgs e)
		{
			System.Windows.Forms.ListViewItem lvi = listView1.FocusedItem;
			if (lvi == null)
			{
				return;
			}
			MailItem mi = (MailItem)lvi;

			//受信未完了時にメール全文を受信する
			if (mi.MailData.DataStream == null)
			{
				mi.MailData.BodyDataReceive += new TKMP.Net.DataReceiveHandler(MailData_DataReceive);
				bool GetFlag = mi.MailData.ReadBody();
				mi.MailData.BodyDataReceive -= new TKMP.Net.DataReceiveHandler(MailData_DataReceive);
				if (!GetFlag)
				{
					return;
				}
			}

			//リーダークラスへ受信したメールデータを渡す
			TKMP.Reader.MailReader re = new TKMP.Reader.MailReader(mi.MailData.DataStream, false);

			//添付ファイルの情報を表示
			string s = string.Empty;
			if (re.FileCount != 0)
			{
				s += "-------------------------添付ファイル---------------------\r\n";

				foreach (TKMP.Reader.File f in re.FileCollection)
				{
					s += f.FileName + "   (" + f.FileSize.ToString() + "byte)\r\n";
				}
				s += "----------------------------------------------------------\r\n";
			}

			//メールの本文をテキストボックスに表示
			this.textBox1.Text = s + re.MainText;


		}
		/// <summary>
		/// データ受信中に発生するイベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MailData_DataReceive(object sender, TKMP.Net.DataReceiveArgs e)
		{
			TKMP.Net.MailData Data = (TKMP.Net.MailData)sender;
			statusBarPanel2.Text = "受信中 " + e.ReadLength.ToString() + " / " + Data.Length.ToString();
			statusStrip1.Refresh();
		}
		/// <summary>
		/// メッセージの受信イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void PopClient_MessageReceive(object sender, TKMP.Net.MessageArgs e)
		{
			if (this.InvokeRequired)
			{
				this.Invoke(new TKMP.Net.MessageReceiveHandler(PopClient_MessageReceive), new object[] { sender, e });
				return;
			}
			richTextBox1.SelectionColor = System.Drawing.Color.Blue;
			richTextBox1.AppendText(e.Message);
			richTextBox1.Focus();
			richTextBox1.Refresh();
		}
		/// <summary>
		/// メッセージの送信イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void PopClient_MessageSend(object sender, TKMP.Net.MessageArgs e)
		{
			if (this.InvokeRequired)
			{
				this.Invoke(new TKMP.Net.MessageSendHandler(PopClient_MessageSend), new object[] { sender, e });
				return;
			}
			richTextBox1.SelectionColor = System.Drawing.Color.Red;
			richTextBox1.AppendText(e.Message);
			richTextBox1.Focus();
			richTextBox1.Refresh();
		}

		/// <summary>
		/// 証明書の内容を検証します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void PopClient_CertificateValidation(object sender, TKMP.Net.CertificateValidationArgs e)
		{
			e.Cancel = false; //全ての証明書を信用します
		}


	}
	/// <summary>
	/// メール情報を保持しているリストビューアイテム
	/// </summary>
	public class MailItem : System.Windows.Forms.ListViewItem
	{
		/// <summary>
		/// メール情報
		/// </summary>
		public readonly TKMP.Net.MailData MailData;
		/// <summary>
		/// コンストラクタ　作成時にメール情報を登録する
		/// </summary>
		/// <param name="MailData"></param>
		public MailItem(TKMP.Net.MailData MailData)
		{
			//プライベート情報として保存
			this.MailData = MailData;

			//ヘッダーの受信を行う
			if (!this.MailData.ReadHeader())
			{
				return;
			}
			//リーダークラスへ受信したストリームを渡す
			TKMP.Reader.MailReader re = new TKMP.Reader.MailReader(this.MailData.HeaderStream, false);
			//リストビューへ情報を表示
			this.Text = new TKMP.Reader.Header.DateTime(re).Value.ToString("yyyy/MM/dd HH:mm");
			this.SubItems.Add(re.HeaderCollection["From"]);
			this.SubItems.Add(re.HeaderCollection["Subject"]);
		}

	}
}